@@ -1,7 +1,7 @@
 /*
   This file is part of LilyPond, the GNU music typesetter.
 
-  Copyright (C) 1997--2015 Han-Wen Nienhuys <hanwen@xs4all.nl>
+  Copyright (C) 1997--2020 Han-Wen Nienhuys <hanwen@xs4all.nl>
 
   LilyPond is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,13 +20,6 @@
 #include "main.hh"
 #include "lily-guile.hh"
 
-#include <cassert>
-#include <clocale>
-#include <cstring>
-#include <cerrno>
-#include <cstdio>
-using namespace std;
-
 #include <unistd.h>
 #include <sys/types.h>
 #include "config.hh"
@@ -40,6 +33,9 @@ using namespace std;
 #if HAVE_GETTEXT
 #include <libintl.h>
 #endif
+#ifdef __MINGW32__
+#include <windows.h>
+#endif
 
 #include "all-font-metrics.hh"
 #include "file-name.hh"
@@ -53,44 +49,27 @@ using namespace std;
 #include "program-option.hh"
 #include "relocate.hh"
 #include "string-convert.hh"
-#include "version.hh"
 #include "warn.hh"
 #include "lily-imports.hh"
 
+#if GS_API
+#include <ghostscript/iapi.h>
+#endif
+
+#include <cassert>
+#include <cerrno>
+#include <clocale>
+#include <cstdio>
+#include <cstring>
+
 /*
  * Global options that can be overridden through command line.
+ * Most variables are defined in file `global-vars.cc'.
  */
 
-/* Names of header fields to be dumped to a separate file. */
-//vector<string> dump_header_fieldnames_global; // moved to global-data.cc
-
-/* Name of initialisation file. */
-//string init_name_global; // moved to global-data.cc
-
-/* Output formats to generate.  */
-//string output_format_global = ""; // moved to global-data.cc
-
-/* Current output name. */
-//string output_name_global; // moved to global-data.cc
-
-/* Run in safe mode? */
-//bool be_safe_global = false; // moved to global-data.cc
-
 /* Provide URI links to the original file */
 bool point_and_click_global = true;
 
-/* Scheme code to execute before parsing, after .scm init.
-   This is where -e arguments are appended to.  */
-//string init_scheme_code_global; // moved to global-data.cc
-//string init_scheme_variables_global; // moved to global-data.cc
-
-//bool relocate_binary = true; // moved to global-data.cc
-
-/*
- * Miscellaneous global stuff.
- */
-//File_path global_path; // moved to global-data.cc
-
 /*
  * File globals.
  */
@@ -100,7 +79,7 @@ static char const *AUTHORS
     "  Jan Nieuwenhuizen <janneke@gnu.org>\n";
 
 static char const *PROGRAM_NAME = "lilypond";
-static char const *PROGRAM_URL = "http://lilypond.org";
+static char const *PROGRAM_URL = "https://lilypond.org";
 
 static char const *NOTICE
   = _i ("This program is free software.  It is covered by the GNU General Public\n"
@@ -124,13 +103,8 @@ static char const *WARRANTY
         "the Free Software Foundation, Inc., 59 Temple Place - Suite 330,\n"
         "Boston, MA 02111-1307, USA.\n");
 
-/* Where the init files live.  Typically:
-   LILYPOND_DATADIR = /usr/share/lilypond
-*/
-//string lilypond_datadir; // moved to global-data.cc
-
 /* The jail specification: USER, GROUP, JAIL, DIR. */
-string jail_spec;
+std::string jail_spec;
 
 /*  The option parser */
 static Getopt_long *option_parser = 0;
@@ -147,72 +121,108 @@ static Long_option_init options_static[]
 =
 {
   {
+    _i ("FORMATs"), "formats", 'f',
+    _i ("dump FORMAT,...  Also as separate options:")
+  },
+  {
+    0, "pdf", 0,
+    _i ("generate PDF files (default)")
+  },
+  {
+    0, "svg", 0,
+    _i ("generate SVG files ")
+  },
+  {
+    0, "png", 0,
+    _i ("generate PNG files ")
+  },
+  {
+    0, "ps", 0,
+    _i ("generate PostScript files")
+  },
+  {
+    0, "eps", 'E',
+    _i ("generate Encapsulated PostScript files")
+  },
+  {
+    _i ("KEY"), "pspdfopt", 'O',
+    _i ("set ps/pdf optimization to KEY, which is either\n"
+        "'size' (default), 'TeX', or 'TeX-GS'")
+  },
+  {
     _i ("SYM[=VAL]"), "define-default", 'd',
-    _i ("set Scheme option SYM to VAL (default: #t).\n"
-    "Use -dhelp for help.")
+    _i ("set Scheme option SYM to VAL (default: #t);\n"
+        "use -dhelp for help")
+  },
+  {
+    _i ("EXPR"), "evaluate", 'e',
+    _i ("evaluate scheme code")
   },
-
-  {_i ("EXPR"), "evaluate", 'e', _i ("evaluate scheme code")},
   /* Bug in option parser: --output =foe is taken as an abbreviation
      for --output-format.  */
-  {_i ("FORMATs"), "formats", 'f', _i ("dump FORMAT,...  Also as separate options:")},
-  {0, "pdf", 0, _i ("generate PDF (default)")},
-  {0, "png", 0, _i ("generate PNG")},
-  {0, "ps", 0, _i ("generate PostScript")},
-  {0, "bigpdfs", 'b', _i("generate big PDF files")},
-  {0, "help", 'h', _i ("show this help and exit")},
   {
-    _i ("FIELD"), "header", 'H', _i ("dump header field FIELD to file\n"
-    "named BASENAME.FIELD")
+    0, "help", 'h',
+    _i ("show this help and exit")
+  },
+  {
+    _i ("FIELD"), "header", 'H',
+    _i ("dump \\header field FIELD to file\n"
+        "named BASENAME.FIELD")
+  },
+  {
+    _i ("DIR"), "include", 'I',
+    _i ("append DIR to search path")
+  },
+  {
+    _i ("FILE"), "init", 'i',
+    _i ("use FILE as init file")
   },
-  {_i ("DIR"), "include", 'I', _i ("add DIR to search path")},
-  {_i ("FILE"), "init", 'i', _i ("use FILE as init file")},
 #if HAVE_CHROOT
   {
-    _i ("USER, GROUP, JAIL, DIR"), "jail", 'j', _i ("chroot to JAIL, become USER:GROUP\n"
-    "and cd into DIR")
+    _i ("USER,GROUP,JAIL,DIR"), "jail", 'j',
+    _i ("chroot to JAIL, become USER:GROUP\n"
+        "and cd into DIR")
   },
 #endif
   {
-    _i ("LOGLEVEL"), "loglevel", 'l', _i ("print log messages according to"
-    " LOGLEVEL.  Possible values are:\n"
-    "NONE, ERROR, WARNING, BASIC, PROGRESS, INFO (default) and DEBUG.")
+    _i ("LOGLEVEL"), "loglevel", 'l',
+    _i ("print log messages according to LOGLEVEL,\n"
+        "which is either NONE, ERROR, WARNING,\n"
+        "BASIC, PROGRESS, INFO (default), or DEBUG")
+  },
+  {
+    _i ("FILE"), "output", 'o',
+    _i ("write output to FILE (suffix will be added)\n"
+        "or to FOLDER, in which case the file name\n"
+        "will be taken from the input file.")
+  },
+  {
+    0, "relocate", 0,
+    _i ("(ignored)")
+  },
+  {
+    0, "silent", 's',
+    _i ("no progress, only error messages\n"
+        "(equivalent to --loglevel=ERROR)")
+  },
+  {
+    0, "version", 'v',
+    _i ("show version number and exit")
+  },
+  {
+    0, "verbose", 'V',
+    _i ("be verbose (equivalent to --loglevel=DEBUG)")
+  },
+  {
+    0, "warranty", 'w',
+    _i ("show warranty and copyright")
   },
-  {_i ("FILE"), "output", 'o', _i ("write output to FILE (suffix will be added)")},
-  {0, "relocate", 0, _i ("relocate using directory of lilypond program")},
-  {0, "silent", 's', _i ("no progress, only error messages (equivalent to loglevel=ERROR)")},
-  {0, "version", 'v', _i ("show version number and exit")},
-  {0, "verbose", 'V', _i ("be verbose (equivalent to loglevel=DEBUG)")},
-  {0, "warranty", 'w', _i ("show warranty and copyright")},
   {0, 0, 0, 0}
 };
 
-char const *LILYPOND_DATADIR = PACKAGE_DATADIR "/" TOPLEVEL_VERSION;
-
-/* x86 defaults to using 80-bit extended precision arithmetic. This can cause
-   problems because the truncation from 80 bits to 64 bits can occur in
-   unpredictable places. To get around this, we tell the x87 FPU to use only
-   double precision. Note that this is not needed for x86_64 because that uses
-   the SSE unit by default instead of the x87 FPU. */
-#if ((defined (__x86__) || defined (__i386__)) \
-  && defined (HAVE_FPU_CONTROL_H) && (HAVE_FPU_CONTROL_H == 1))
-
-#include <fpu_control.h>
-static void
-configure_fpu ()
-{
-  fpu_control_t fpu_control = 0x027f;
-  _FPU_SETCW (fpu_control);
-}
-
-#else
-
-static void
-configure_fpu ()
-{
-}
-
-#endif /* defined(__x86__) || defined(__i386__) */
+using std::map;
+using std::string;
+using std::vector;
 
 static void
 env_var_info (FILE *out, char const *key)
@@ -234,24 +244,23 @@ dir_info (FILE *out)
  */
 {
   fputs ("\n", out);
-  fprintf (out, "LILYPOND_DATADIR=\"%s\"\n", LILYPOND_DATADIR);
-  env_var_info (out, "LILYPONDPREFIX");
   env_var_info (out, "LILYPOND_DATADIR");
-  fprintf (out, "LOCALEDIR=\"%s\"\n", LOCALEDIR);
+  env_var_info (out, "LILYPOND_LOCALEDIR");
+  env_var_info (out, "LILYPOND_RELOCDIR");
 
-  fprintf (out, "\nEffective prefix: \"%s\"\n", lilypond_datadir.c_str ());
+  fputs (_f ("\n"
+             "Effective prefix: '%s'\n",
+             lilypond_datadir).c_str (), out);
 
-  if (relocate_binary)
-    {
-      env_var_info (out, "FONTCONFIG_FILE");
-      env_var_info (out, "FONTCONFIG_PATH");
-      env_var_info (out, "GS_FONTPATH");
-      env_var_info (out, "GS_LIB");
-      env_var_info (out, "GUILE_LOAD_PATH");
-      env_var_info (out, "PANGO_RC_FILE");
-      env_var_info (out, "PANGO_PREFIX");
-      env_var_info (out, "PATH");
-    }
+  env_var_info (out, "FONTCONFIG_FILE");
+  env_var_info (out, "FONTCONFIG_PATH");
+  env_var_info (out, "GS_FONTPATH");
+  env_var_info (out, "GS_LIB");
+  env_var_info (out, "GUILE_LOAD_PATH");
+  env_var_info (out, "PANGO_RC_FILE");
+  env_var_info (out, "PANGO_PREFIX");
+  env_var_info (out, "PATH");
+  fputs ("\n", out);
 }
 
 static void
@@ -261,7 +270,7 @@ copyright ()
  */
 {
   /* Do not update the copyright years here, run `make grand-replace'  */
-  printf ("%s", (_f ("Copyright (c) %s by\n%s  and others.", "1996--2015",
+  printf ("%s", (_f ("Copyright (c) %s by\n%s  and others.", "1996--2021",
                      AUTHORS).c_str ()));
   printf ("\n");
 }
@@ -315,9 +324,9 @@ LY_DEFINE (ly_usage, "ly:usage",
      "Report bugs in English via %s",
      or if there is a LilyPond users list or forum in your language
      "Report bugs in English via %s or in YOUR_LANG via URI"  */
-  printf ("%s", (_f ("Report bugs via %s",
-                     "http://post.gmane.org/post.php?group=gmane.comp.gnu.lilypond.bugs"
-                    ).c_str ()));
+  printf ("%s", (_f ("You found a bug? Please read %s",
+                     "https://lilypond.org/bug-reports.html")
+                 .c_str ()));
   printf ("\n");
   printf ("\n");
   return SCM_UNSPECIFIED;
@@ -334,6 +343,26 @@ warranty ()
   copyright ();
   printf ("\n");
   printf ("%s", (_ (WARRANTY).c_str ()));
+
+#if GS_API
+  printf ("\n");
+  printf ("%s", (_ ("linked against Ghostscript:").c_str ()));
+  printf ("\n");
+
+  void *gs_inst = NULL;
+  gsapi_new_instance (&gs_inst, NULL);
+  gsapi_set_arg_encoding (gs_inst, GS_ARG_ENCODING_UTF8);
+
+  // gsapi_init_with_args wants modifiable strings, so create local variables
+  // with copies of the content.
+  // (The first argument is the "program name" and is ignored.)
+  char gs[] = "gs";
+  char nodisplay[] = "-dNODISPLAY";
+  char *argv[] = { gs, nodisplay };
+  gsapi_init_with_args (gs_inst, 2, argv);
+  gsapi_exit (gs_inst);
+  gsapi_delete_instance (gs_inst);
+#endif
 }
 
 static void
@@ -345,12 +374,12 @@ prepend_scheme_list (const string &dir, const string &scmlist)
  *    scmlist: The Scheme list onto which to prepend the directory
  */
 {
-  SCM var = scm_c_lookup (scmlist.c_str());
-  scm_variable_set_x (var, scm_cons (scm_from_locale_string (dir.c_str()),
-				     scm_variable_ref (var)));
+  SCM var = scm_c_lookup (scmlist.c_str ());
+  scm_variable_set_x (var, scm_cons (scm_from_locale_string (dir.c_str ()),
+                                     scm_variable_ref (var)));
   /*  string setcmd =
              "(set! " + scmlist + " (cons \"" + dir + "\" " + scmlist +"))";
-	     scm_c_eval_string (setcmd.c_str());*/
+             scm_c_eval_string (setcmd.c_str());*/
 }
 
 void init_global_tweak_registry ();
@@ -455,41 +484,43 @@ main_with_guile (void *, int, char **)
       %load-path is the symbol Guile searches for .scm files
       %load-compiled-path is the symbol Guile V2 searches for .go files
    */
-   string scm_pct_load_path = "%load-path";
-   string scm_pct_load_compiled_path = "%load-compiled-path";
+  string scm_pct_load_path = "%load-path";
+  string scm_pct_load_compiled_path = "%load-compiled-path";
 
-   prepend_scheme_list (lilypond_datadir, scm_pct_load_path );
-   prepend_scheme_list (lilypond_datadir + "/scm", scm_pct_load_path);
+  prepend_scheme_list (lilypond_datadir, scm_pct_load_path);
+  prepend_scheme_list (lilypond_datadir + "/scm", scm_pct_load_path);
 
-#if (GUILE2)
-   /*
-     Just as ughy - prepend "/scm/out" onto GUILE V2+ %load-compiled-path
-     and set %compile-fallback-path to our scm/out directory
-   */
-   /*
-     %load-compiled-path is the symbol Guile V2 searches for .go files
-   */
-   prepend_scheme_list (lilypond_datadir + "/scm/out",
-			  scm_pct_load_compiled_path);
-   /*
-     %compile-fallback-path is the guile cache root for auto-compiled files
-   */
+#if 0
+  /* this code is dead, but keeping this around until we sort the
+     guile2/3 situation. */
+  /*
+    Just as ughy - prepend "/scm/out" onto GUILE V2+ %load-compiled-path
+    and set %compile-fallback-path to our scm/out directory
+  */
+  /*
+    %load-compiled-path is the symbol Guile V2 searches for .go files
+  */
+  prepend_scheme_list (lilypond_datadir + "/scm/out",
+                       scm_pct_load_compiled_path);
+  /*
+    %compile-fallback-path is the guile cache root for auto-compiled files
+  */
 
-   string scm_pct_fallback_path = "%compile-fallback-path";
-   string ly_scm_go_dir = lilypond_datadir + "/scm/out";
-   //string scm_pct_set_fallback = "(set! " + scm_pct_fallback_path +
-   //  " \"" + lilypond_datadir + "/scm/out\")";
-   //scm_c_eval_string (scm_pct_set_fallback.c_str() );
-   scm_primitive_eval
-     (scm_list_3 (scm_from_latin1_symbol ("set!"),
-                  scm_from_latin1_symbol ("%compile-fallback-path"),
-                  scm_from_locale_string (ly_scm_go_dir.c_str())));
+  string scm_pct_fallback_path = "%compile-fallback-path";
+  string ly_scm_go_dir = lilypond_datadir + "/scm/out";
+  //string scm_pct_set_fallback = "(set! " + scm_pct_fallback_path +
+  //  " \"" + lilypond_datadir + "/scm/out\")";
+  //scm_c_eval_string (scm_pct_set_fallback.c_str() );
+  scm_primitive_eval
+  (scm_list_3 (scm_from_latin1_symbol ("set!"),
+               scm_from_latin1_symbol ("%compile-fallback-path"),
+               scm_from_locale_string (ly_scm_go_dir.c_str ())));
 #endif
 
   if (is_loglevel (LOG_DEBUG))
     dir_info (stderr);
 
-  init_scheme_variables_global = "(list " + init_scheme_variables_global + ")";
+  init_scheme_variables_global = "(" + init_scheme_variables_global + ")";
   init_scheme_code_global = "(begin " + init_scheme_code_global + ")";
 
   ly_c_init_guile ();
@@ -497,7 +528,6 @@ main_with_guile (void *, int, char **)
   init_fontconfig ();
 
   init_freetype ();
-  ly_reset_all_fonts ();
 
   /*
      We accept multiple independent music files on the command line to
@@ -531,8 +561,8 @@ main_with_guile (void *, int, char **)
   // SCM err_handling_mod = scm_c_resolve_module ("system repl error-handling");
   // SCM call_with_error_handling = scm_c_module_lookup (err_handling_mod, "call-with-error-handling");
   // SCM result = scm_call_1 (
-  // 			   scm_variable_ref (call_with_error_handling),
-  // 			   scm_call_1 (ly_lily_module_constant ("lilypond-main"), files));
+  //                       scm_variable_ref (call_with_error_handling),
+  //                       scm_call_1 (ly_lily_module_constant ("lilypond-main"), files));
 
   Lily::lilypond_main (files);
 
@@ -550,31 +580,31 @@ setup_localisation ()
 {
 #if HAVE_GETTEXT
   /* Enable locales */
+#if !defined (__MINGW32__) || defined (_UCRT)
   setlocale (LC_ALL, "");
+#else
+  // Workaround for MinGW UTF-8 locale settings issue:
+  // `setlocale (LC_ALL, "")` in msvcrt.dll sets user-defined ANSI code page
+  // (i.e. 437 for English, 932 for Japanse, etc.)
+  // even if process code page is UTF-8 (i.e. 65001 for all languages).
+  // With this setting, outputting UTF-8 string becomes garbled.
+  // So we get the process code page with `GetACP ()` and,
+  // if it is UTF-8, explicitly set locale's code page to UTF-8 (i.e. 65001).
+  // If Universal CRT (UCRT, newer than msvcrt.dll) is used
+  // (i.e. `_UCRT' is defined), no such workaround is needed.
+  if (GetACP () == CP_UTF8)
+    setlocale (LC_ALL, ".65001");
+  else
+    setlocale (LC_ALL, "");
+#endif
 
   /* FIXME: check if this is still true.
      Disable localisation of float values. */
   setlocale (LC_NUMERIC, "C");
 
-#if GUILEV2
-  // In order not to have this porting aid backfire to GUILE1 usage,
-  // this is only compiled in the GUILEV2 version.  It should
-  // eventually be replaced with proper multibyte communication with
-  // GUILE2, but in the mean time it seems that this is the least
-  // invasive path to get comparable results between the
-  // not-really-multibyte-supporting GUILE1 and GUILE2
-
-  /* Disable character sets */
-  setlocale (LC_CTYPE, "C");
-  /* But our text domain is in UTF-8 */
-  bind_textdomain_codeset ("lilypond", "UTF-8");
-#endif
-
-  string localedir = LOCALEDIR;
-  if (char const *env = getenv ("LILYPOND_LOCALEDIR"))
-    localedir = env;
-
-  bindtextdomain ("lilypond", localedir.c_str ());
+  // we temporarily use the compile-time value for the locale
+  // until we get the final directory location
+  bindtextdomain ("lilypond", LOCALEDIR);
   textdomain ("lilypond");
 #endif
 }
@@ -606,17 +636,64 @@ parse_argv (int argc, char **argv)
     {
       switch (opt->shortname_char_)
         {
+        case 'f':
+          {
+            vector<string> components
+              = string_split (option_parser->optional_argument_str0_, ',');
+            bool wants_svg = 0;
+            for (vsize i = 0; i < components.size (); i++)
+              {
+                string format = (components[i]);
+                if (format == "svg")
+                  {
+                    wants_svg = 1;
+                    init_scheme_variables_global += "(backend . svg)\n";
+                  }
+                if ((i > 0) && (wants_svg))
+                  {
+                    warning (_ ("SVG backend requested; other formats will be ignored."));
+                    wants_svg = 0;
+                  }
+                add_output_format (format);
+              }
+          }
+          break;
+
         case 0:
           if (string (opt->longname_str0_) == "pdf"
               || string (opt->longname_str0_) == "png"
               || string (opt->longname_str0_) == "ps")
             add_output_format (opt->longname_str0_);
+          else if (string (opt->longname_str0_) == "svg")
+            init_scheme_variables_global += "(backend . svg)\n";
           else if (string (opt->longname_str0_) == "relocate")
-            relocate_binary = true;
+            warning (_ ("The --relocate option is no longer relevant."));
           break;
 
-        case 'b':
-          bigpdfs = true;
+        case 'E':
+          add_output_format ("ps");
+          init_scheme_variables_global
+          += "(backend . eps)\n(aux-files . #f)\n";
+          break;
+
+        case 'O':
+          {
+            string arg (option_parser->optional_argument_str0_);
+            if (arg == "size")
+              init_scheme_variables_global
+              += "(music-font-encodings . #f)\n"
+                 "(gs-never-embed-fonts . #f)\n";
+            else if (arg == "TeX-GS")
+              init_scheme_variables_global
+              += "(music-font-encodings . #t)\n"
+                 "(gs-never-embed-fonts . #t)\n";
+            else if (arg == "TeX")
+              init_scheme_variables_global
+              += "(music-font-encodings . #t)\n"
+                 "(gs-never-embed-fonts . #f)\n";
+            else
+              programming_error ("Ignoring unknown optimization key");
+          }
           break;
 
         case 'd':
@@ -634,7 +711,7 @@ parse_argv (int argc, char **argv)
               }
 
             init_scheme_variables_global
-            += "(cons \'" + key + " '" + val + ")\n";
+            += "(" + key + " . " + val + ")\n";
           }
           break;
 
@@ -662,15 +739,6 @@ parse_argv (int argc, char **argv)
           exit (0);
           break;
 
-        case 'f':
-          {
-            vector<string> components
-              = string_split (option_parser->optional_argument_str0_, ',');
-            for (vsize i = 0; i < components.size (); i++)
-              add_output_format (components[i]);
-          }
-          break;
-
         case 'H':
           dump_header_fieldnames_global
           .push_back (option_parser->optional_argument_str0_);
@@ -742,6 +810,7 @@ setup_guile_gc_env ()
                "104857600", overwrite);
 }
 
+#if (GUILEV2)
 
 void
 setup_guile_v2_env ()
@@ -750,16 +819,39 @@ setup_guile_v2_env ()
  * Scheme files for Guile V2.
  */
 {
-     sane_putenv("GUILE_AUTO_COMPILE", "0", true);  // disable auto-compile
-     sane_putenv("GUILE_WARN_DEPRECATED",
-                  "detailed", "true");   // set Guile to info re deprecation
-     /*
+  sane_putenv ("GUILE_AUTO_COMPILE", "0", false); // disable auto-compile
+  sane_putenv ("GUILE_WARN_DEPRECATED", "detailed",
+               "true"); // set Guile to info re deprecation
+  /*
         Set root for Guile %compile-fallback to
         Lilypond root for its data.
       */
-     sane_putenv("XDG_CACHE_HOME",
-                  lilypond_datadir, true);
+  sane_putenv ("XDG_CACHE_HOME", lilypond_datadir, true);
+
+  // This reduces the GC overhead during parsing and
+  // initialization. To see if this is a good value, run #(display
+  // (gc-stats)) just before \maininput in init.ly, and check that
+  // it's roughly this value.
+  sane_putenv ("GC_INITIAL_HEAP_SIZE", "40M", false);
+
+  /*
+    Empirically, multithreaded GC doesn't change wall time. It just
+    adds another thread that burns 30% of the time.
+
+    David K mentions: "I think that this may be due to both/either our
+    use of mark hooks and of finalisers for calling destructors.
+    Either may cause serialisation.  Another serialisation is because
+    Guile itself switches BGC to Java mode where finalised objects can
+    no longer be marked (or something like that: the exact semantics I
+    do not remember).  And of course the C++ free store still has to
+    do its full job.
+  */
+  sane_putenv ("GC_NPROCS", "1", false);
+
+  // Use less CPU for GC, at the expense of memory.
+  sane_putenv ("GC_FREE_SPACE_DIVISOR", "1", false);
 }
+#endif
 
 void
 setup_guile_env ()
@@ -767,10 +859,10 @@ setup_guile_env ()
  * Set up environment variables relevant to Scheme
  */
 {
-
-  setup_guile_gc_env ();  // configure garbage collector
 #if (GUILEV2)
   setup_guile_v2_env ();  // configure Guile V2 behaviour
+#else
+  setup_guile_gc_env ();  // configure garbage collector
 #endif
 }
 
@@ -786,7 +878,6 @@ main (int argc, char **argv, char **envp)
  * envp:   Point to vector of OS environment variables
  */
 {
-  configure_fpu ();
   /*
     Process environment variables
   */
@@ -809,25 +900,16 @@ main (int argc, char **argv, char **envp)
 
   setup_paths (argv[0]);
   setup_guile_env ();  // set up environment variables to pass into Guile API
+
+#if !GS_API
+  // Let Guile know whether the Ghostscript API is not available.
+  init_scheme_variables_global += "(gs-api . #f)\n";
+#endif
+
   /*
    * Start up Guile API using main_with_guile as a callback.
    */
-#if (GUILEV2)
- /* Debugging aid.
-    Set it on by default for Guile V2
-    while migration in progress.
- */
-  try
-    {
-      scm_boot_guile (argc, argv, main_with_guile, 0);
-    }
-  catch (exception e)
-    {
-      error (_f ("exception caught: %s", e.what ()));
-    };
-#else
   scm_boot_guile (argc, argv, main_with_guile, 0);
-#endif
 
   /* Only reachable if GUILE exits.  That is an error.  */
   return 1;
